<?php

// autoload_static.php @generated by Composer

namespace Composer\Autoload;

class ComposerStaticInit08b1cb29d17a6da806c716ad0b7e2455
{
    public static $prefixLengthsPsr4 = array (
        'P' => 
        array (
            'PHPMailer\\PHPMailer\\' => 20,
        ),
    );

    public static $prefixDirsPsr4 = array (
        'PHPMailer\\PHPMailer\\' => 
        array (
            0 => __DIR__ . '/../..' . '/src',
        ),
    );

    public static $classMap = array (
        'Composer\\InstalledVersions' => __DIR__ . '/..' . '/composer/InstalledVersions.php',
    );

    public static function getInitializer(ClassLoader $loader)
    {
        return \Closure::bind(function () use ($loader) {
            $loader->prefixLengthsPsr4 = ComposerStaticInit08b1cb29d17a6da806c716ad0b7e2455::$prefixLengthsPsr4;
            $loader->prefixDirsPsr4 = ComposerStaticInit08b1cb29d17a6da806c716ad0b7e2455::$prefixDirsPsr4;
            $loader->classMap = ComposerStaticInit08b1cb29d17a6da806c716ad0b7e2455::$classMap;

        }, null, ClassLoader::class);
    }
}
