<?php return array(
    'root' => array(
        'name' => 'cacti/cacti',
        'pretty_version' => '1.2.x-dev',
        'version' => '1.2.9999999.9999999-dev',
        'reference' => '565b16dce89591c2408c8aa491a3003ae8d28b4b',
        'type' => 'composer-plugin',
        'install_path' => __DIR__ . '/../../../',
        'aliases' => array(),
        'dev' => false,
    ),
    'versions' => array(
        'cacti/cacti' => array(
            'pretty_version' => '1.2.x-dev',
            'version' => '1.2.9999999.9999999-dev',
            'reference' => '565b16dce89591c2408c8aa491a3003ae8d28b4b',
            'type' => 'composer-plugin',
            'install_path' => __DIR__ . '/../../../',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ezyang/htmlpurifier' => array(
            'pretty_version' => 'v4.19.0',
            'version' => '4.19.0.0',
            'reference' => 'b287d2a16aceffbf6e0295559b39662612b77fcf',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ezyang/htmlpurifier',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'paragonie/constant_time_encoding' => array(
            'pretty_version' => 'v2.8.2',
            'version' => '2.8.2.0',
            'reference' => 'e30811f7bc69e4b5b6d5783e712c06c8eabf0226',
            'type' => 'library',
            'install_path' => __DIR__ . '/../paragonie/constant_time_encoding',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'paragonie/random_compat' => array(
            'pretty_version' => 'v9.99.100',
            'version' => '9.99.100.0',
            'reference' => '996434e5492cb4c3edcb9168db6fbb1359ef965a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../paragonie/random_compat',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'phpseclib/phpseclib' => array(
            'pretty_version' => '3.0.52',
            'version' => '3.0.52.0',
            'reference' => '2adaefc83df2ec548558307690f376dd7d4f4fce',
            'type' => 'library',
            'install_path' => __DIR__ . '/../phpseclib/phpseclib',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
    ),
);
